@@ -17,12 +17,10 @@ import * as nbformat from '@jupyterlab/nbformat';
 const CELL_TYPE_NA = 'na';
 const CELL_TYPE_NA_LABEL = '-';
 
-export const CELL_TYPE_PREPROCESSOR = 'preprocessor';
-export const CELL_TYPE_POSTPROCESSOR = 'postprocessor';
+export const CELL_TYPE_PREDICT = 'predict';
 export const CELL_TYPE_REQUIREMENTS = 'requirements';
 export const CELL_TYPE_CVAT_INFO = 'cvat-info';
-export const CELL_TYPE_CVAT_PREPROCESSOR = 'cvat-preprocessor';
-export const CELL_TYPE_CVAT_POSTPROCESSOR = 'cvat-postprocessor';
+export const CELL_TYPE_CVAT_INVOKE = 'cvat-invoke';
 const CELL_TYPES: any[] = [
     {
         value: CELL_TYPE_NA,
@@ -37,21 +35,15 @@ const CELL_TYPES: any[] = [
         chipColor: '84ffff'
     },
     {
-        value: CELL_TYPE_PREPROCESSOR,
-        label: 'Pre processor',
-        helpText: 'The code in this cell will be parsed as preprocessor for the predict function of deployed model.',
+        value: CELL_TYPE_PREDICT,
+        label: 'Predict',
+        helpText: 'The code in this cell will be parsed as the handler of the predict API on proxy.',
         chipColor: 'b9f6ca'
     },
-    {
-        value: CELL_TYPE_POSTPROCESSOR,
-        label: 'Post processor',
-        helpText: 'The code in this cell will be parsed as postprocessor for the predict function of deployed model.',
-        chipColor: '69f0ae'
-    },
     {
         value: 'functions',
         label: 'Extra functions',
-        helpText: 'The code in this cell will be parsed as referenced function for the preprocessor or postprocessor.',
+        helpText: 'The code in this cell will be parsed as referenced function for the predict or cvat_invoke functions.',
         chipColor: 'ccff90'
     },
     {
@@ -61,16 +53,10 @@ const CELL_TYPES: any[] = [
         chipColor: 'c6ff00'
     },
     {
-        value: CELL_TYPE_CVAT_PREPROCESSOR,
-        label: 'CVAT Pre processor',
-        helpText: 'The code in this cell will be parsed as CVAT preprocessor for the predict function of deployed model.',
+        value: CELL_TYPE_CVAT_INVOKE,
+        label: 'CVAT invoke',
+        helpText: 'The code in this cell will be parsed as the handler of the cvat_invoke API on proxy.',
         chipColor: 'f4ff81'
-    },
-    {
-        value: CELL_TYPE_CVAT_POSTPROCESSOR,
-        label: 'CVAT Post processor',
-        helpText: 'The code in this cell will be parsed as CVAT postprocessor for the predict function of deployed model.',
-        chipColor: 'eeff41'
     }
 ];
 
@@ -101,17 +87,13 @@ export const RESERVED_CELL_NAMES_CHIP_COLOR = CELL_TYPES
         return obj;
     } ,{});
 
-const PREPROCESSOR_DEF_REGEX = /\s*def\s+preprocess\s*\(\s*\S+\s*(:\s*\S+\s*)*\)\s*(->\s*\S+\s*)*:\s*/i
-const PREPROCESSOR_DEFAULT_CODE_SNIPPET = `from typing import Dict
-def preprocess(input: Dict) -> Dict:
-    return input`
+const PREDICT_DEF_REGEX = /\s*def\s+predict\s*\((\s*\w+\s*(:\s*\w+\s*)?(=[\w\W]+)?,?){0,2}\)\s*(->\s*\S+\s*)?:\s*/i
+const PREDICT_DEFAULT_CODE_SNIPPET = `from typing import Dict
+from flask import redirect
+def predict(input: Dict, model_predict_url: str) -> Dict:
+    return redirect(model_predict_url)`
 
-const POSTPROCESSOR_DEF_REGEX = /\s*def\s+postprocess\s*\(\s*\S+\s*(:\s*\S+\s*)*\)\s*(->\s*\S+\s*)*:\s*/i
-const POSTPROCESSOR_DEFAULT_CODE_SNIPPET = `from typing import Dict
-def postprocess(input: Dict) -> Dict:
-    return input`
-
-const CVAT_INFO_DEF_REGEX = /\s*def\s+cvat_info\s*\(\s*\S*\s*(:\s*\S+\s*)*(=\s*('|")*\S+('|")*\s*)*\)\s*(->\s*\S+\s*)*:\s*/i
+const CVAT_INFO_DEF_REGEX = /\s*def\s+cvat_info\s*\((\s*\w+\s*(:\s*\w+\s*)?(=[\w\W]+)?,?){0,1}\)\s*(->\s*\S+\s*)?:\s*/i
 const CVAT_INFO_DEFAULT_CODE_SNIPPET = `from typing import Dict
 def cvat_info(inputs: Dict = None) -> Dict:
     return {
@@ -121,14 +103,10 @@ def cvat_info(inputs: Dict = None) -> Dict:
         "description": ""
     }`
 
-const CVAT_PREPROCESSOR_DEF_REGEX = /\s*def\s+cvat_preprocess\s*\(\s*\S+\s*(:\s*\S+\s*)*\)\s*(->\s*\S+\s*)*:\s*/i
-const CVAT_PREPROCESSOR_DEFAULT_CODE_SNIPPET = `from typing import Dict
-def cvat_preprocess(input: Dict) -> Dict:
-    return input`
-
-const CVAT_POSTPROCESSOR_DEF_REGEX = /\s*def\s+cvat_postprocess\s*\(\s*\S+\s*(:\s*\S+\s*)*\)\s*(->\s*\S+\s*)*:\s*/i
-const CVAT_POSTPROCESSOR_DEFAULT_CODE_SNIPPET = `from typing import Dict
-def cvat_postprocess(input: Dict) -> Dict:
+const CVAT_INVOKE_DEF_REGEX = /\s*def\s+cvat_invoke\s*\((\s*\w+\s*(:\s*\w+\s*)?(=[\w\W]+)?,?){0,2}\)\s*(->\s*\S+\s*)?:\s*/i
+const CVAT_INVOKE_DEFAULT_CODE_SNIPPET = `from typing import Dict
+from flask import redirect
+def cvat_invoke(input: Dict, model_predict_url: str) -> Dict:
     return []`
 
 const enum CellCodeErrors {
@@ -185,35 +163,27 @@ export class CellMetadataEditor extends React.Component<IProps, IState> {
     }
 
     isTagged = (tag: string): boolean => {
-        if(tag === CELL_TYPE_PREPROCESSOR) {
-            return this.transformerStates.isPreprocessorTagged;
-        } else if(tag === CELL_TYPE_POSTPROCESSOR) {
-            return this.transformerStates.isPostprocessorTagged;
+        if(tag === CELL_TYPE_PREDICT) {
+            return this.transformerStates.isPredictTagged;
         } else if(tag === CELL_TYPE_REQUIREMENTS) {
             return this.transformerStates.isRequirementsTagged;
         } else if(tag === CELL_TYPE_CVAT_INFO) {
             return this.transformerStates.isCVATInfoTagged;
-        } else if(tag === CELL_TYPE_CVAT_PREPROCESSOR) {
-            return this.transformerStates.isCVATPreprocessorTagged;
-        } else if(tag === CELL_TYPE_CVAT_POSTPROCESSOR) {
-            return this.transformerStates.isCVATPostprocessorTagged;
+        } else if(tag === CELL_TYPE_CVAT_INVOKE) {
+            return this.transformerStates.isCVATInvokeTagged;
         }
         return false;
     }
 
     updateLocalTransformerStates = (tag: string, tagged: boolean): void => {
-        if(tag === CELL_TYPE_PREPROCESSOR) {
-            this.transformerStates.isPreprocessorTagged = tagged;
-        } else if(tag === CELL_TYPE_POSTPROCESSOR) {
-            this.transformerStates.isPostprocessorTagged = tagged;
+        if(tag === CELL_TYPE_PREDICT) {
+            this.transformerStates.isPredictTagged = tagged;
         } else if(tag === CELL_TYPE_REQUIREMENTS) {
             this.transformerStates.isRequirementsTagged = tagged;
         } else if(tag === CELL_TYPE_CVAT_INFO) {
             this.transformerStates.isCVATInfoTagged = tagged;
-        } else if(tag === CELL_TYPE_CVAT_PREPROCESSOR) {
-            this.transformerStates.isCVATPreprocessorTagged = tagged;
-        } else if(tag === CELL_TYPE_CVAT_POSTPROCESSOR) {
-            this.transformerStates.isCVATPostprocessorTagged = tagged;
+        } else if(tag === CELL_TYPE_CVAT_INVOKE) {
+            this.transformerStates.isCVATInvokeTagged = tagged;
         }
     }
 
@@ -232,16 +202,12 @@ export class CellMetadataEditor extends React.Component<IProps, IState> {
     checkCellCodeDefinitionConfirmed = (notebook: Notebook, cellIndex: number, tag: string) => {
         console.log("checkCellCodeDefinitionConfirmed");
         let regex = null;
-        if(tag == CELL_TYPE_PREPROCESSOR) {
-            regex = PREPROCESSOR_DEF_REGEX;
-        } else if(tag == CELL_TYPE_POSTPROCESSOR) {
-            regex = POSTPROCESSOR_DEF_REGEX;
+        if(tag == CELL_TYPE_PREDICT) {
+            regex = PREDICT_DEF_REGEX;
         } else if(tag == CELL_TYPE_CVAT_INFO) {
             regex = CVAT_INFO_DEF_REGEX;
-        } else if(tag == CELL_TYPE_CVAT_PREPROCESSOR) {
-            regex = CVAT_PREPROCESSOR_DEF_REGEX;
-        } else if(tag == CELL_TYPE_CVAT_POSTPROCESSOR) {
-            regex = CVAT_POSTPROCESSOR_DEF_REGEX;
+        } else if(tag == CELL_TYPE_CVAT_INVOKE) {
+            regex = CVAT_INVOKE_DEF_REGEX;
         }
         if(regex) {
             if(CellUtils.isCellVoid(notebook, cellIndex)) {
@@ -264,30 +230,17 @@ export class CellMetadataEditor extends React.Component<IProps, IState> {
 
     updateCurrentCellTag = async (value: string) => {
         if(value !== this.state.transformerTag) {
-            this.updateLocalTransformerStates(this.state.transformerTag, false);
-            this.updateLocalTransformerStates(value, true);
-            issueTransformerStatesChange(this.props.notebookPanel.id, this.props.cellId, this.transformerStates);
-            if (RESERVED_CELL_NAMES.includes(value)) {
-                this.setState({ transformerTag: value });
-            } else if(CELL_TYPE_NA === value) {
-                this.setState({ transformerTag: CELL_TYPE_NA });
-            }
-
             let notebook: Notebook = this.props.notebookPanel.content;
             let currentCellType: string = this.props.notebookPanel.model.cells.get(notebook.activeCellIndex).type;
             let targetCellType: string = currentCellType;
-            if(value == CELL_TYPE_PREPROCESSOR || value == CELL_TYPE_POSTPROCESSOR || value == CELL_TYPE_CVAT_INFO || value == CELL_TYPE_CVAT_PREPROCESSOR || value == CELL_TYPE_CVAT_POSTPROCESSOR) {
+            if(value == CELL_TYPE_PREDICT || value == CELL_TYPE_CVAT_INFO || value == CELL_TYPE_CVAT_INVOKE) {
                 let confirmedResult = this.checkCellCodeDefinitionConfirmed(notebook, notebook.activeCellIndex, value);
                 if(! confirmedResult.confirmed) {
-                    let codeSnippet = PREPROCESSOR_DEFAULT_CODE_SNIPPET;
-                    if(value == CELL_TYPE_POSTPROCESSOR) {
-                        codeSnippet = POSTPROCESSOR_DEFAULT_CODE_SNIPPET;
-                    } else if(value == CELL_TYPE_CVAT_INFO) {
+                    let codeSnippet = PREDICT_DEFAULT_CODE_SNIPPET;
+                    if(value == CELL_TYPE_CVAT_INFO) {
                         codeSnippet = CVAT_INFO_DEFAULT_CODE_SNIPPET;
-                    } else if(value == CELL_TYPE_CVAT_PREPROCESSOR) {
-                        codeSnippet = CVAT_PREPROCESSOR_DEFAULT_CODE_SNIPPET;
-                    } else if(value == CELL_TYPE_CVAT_POSTPROCESSOR) {
-                        codeSnippet = CVAT_POSTPROCESSOR_DEFAULT_CODE_SNIPPET;
+                    } else if(value == CELL_TYPE_CVAT_INVOKE) {
+                        codeSnippet = CVAT_INVOKE_DEFAULT_CODE_SNIPPET;
                     }
                     if(confirmedResult.error == CellCodeErrors.VOID) {
                         CellUtils.injectCodeAtIndex(notebook, notebook.activeCellIndex, codeSnippet);
@@ -300,6 +253,7 @@ export class CellMetadataEditor extends React.Component<IProps, IState> {
                             dialogCode: codeSnippet
                         });
                         this.toggleTagsEditorDialog();
+                        return;
                     }
                 }
                 targetCellType = 'code';
@@ -307,6 +261,15 @@ export class CellMetadataEditor extends React.Component<IProps, IState> {
                 targetCellType = 'raw';
             }
 
+            this.updateLocalTransformerStates(this.state.transformerTag, false);
+            this.updateLocalTransformerStates(value, true);
+            issueTransformerStatesChange(this.props.notebookPanel.id, this.props.cellId, this.transformerStates);
+            if (RESERVED_CELL_NAMES.includes(value)) {
+                this.setState({ transformerTag: value });
+            } else if(CELL_TYPE_NA === value) {
+                this.setState({ transformerTag: CELL_TYPE_NA });
+            }
+
             if(currentCellType != targetCellType) {
                 console.log("Keep transformer metadata in cell metadata and replace cell widget...");
                 let cellMetadata = {
@@ -329,7 +292,7 @@ export class CellMetadataEditor extends React.Component<IProps, IState> {
         console.log("saveCellTagInNotebookFile");
         let value = this.state.transformerTag;
 
-            if(value == CELL_TYPE_PREPROCESSOR || value == CELL_TYPE_POSTPROCESSOR || value == CELL_TYPE_CVAT_INFO || value == CELL_TYPE_CVAT_PREPROCESSOR || value == CELL_TYPE_CVAT_POSTPROCESSOR) {
+        if(value == CELL_TYPE_PREDICT || value == CELL_TYPE_CVAT_INFO || value == CELL_TYPE_CVAT_INVOKE) {
             let notebook: Notebook = this.props.notebookPanel.content;
             let confirmedResult = this.checkCellCodeDefinitionConfirmed(notebook, notebook.activeCellIndex, value);
             if(! confirmedResult.confirmed) {
@@ -345,15 +308,11 @@ export class CellMetadataEditor extends React.Component<IProps, IState> {
                     return;
                 }
                 if(confirmedResult.error == CellCodeErrors.FUNC_DEF_ERROR) {
-                    let codeSnippet = PREPROCESSOR_DEFAULT_CODE_SNIPPET;
-                    if(value == CELL_TYPE_POSTPROCESSOR) {
-                        codeSnippet = POSTPROCESSOR_DEFAULT_CODE_SNIPPET;
-                    } else if(value == CELL_TYPE_CVAT_INFO) {
+                    let codeSnippet = PREDICT_DEFAULT_CODE_SNIPPET;
+                    if(value == CELL_TYPE_CVAT_INFO) {
                         codeSnippet = CVAT_INFO_DEFAULT_CODE_SNIPPET;
-                    } else if(value == CELL_TYPE_CVAT_PREPROCESSOR) {
-                        codeSnippet = CVAT_PREPROCESSOR_DEFAULT_CODE_SNIPPET;
-                    } else if(value == CELL_TYPE_CVAT_POSTPROCESSOR) {
-                        codeSnippet = CVAT_POSTPROCESSOR_DEFAULT_CODE_SNIPPET;
+                    } else if(value == CELL_TYPE_CVAT_INVOKE) {
+                        codeSnippet = CVAT_INVOKE_DEFAULT_CODE_SNIPPET;
                     }
                     let dialogTitle = "Function %s is not well defined!".replace(/%s/i, value);
                     let dialogContent = "Please define function %s as follows:".replace(/%s/i, value);
@@ -369,7 +328,7 @@ export class CellMetadataEditor extends React.Component<IProps, IState> {
         }
 
         let isDuplicated = false;
-        if(value == CELL_TYPE_PREPROCESSOR || value == CELL_TYPE_POSTPROCESSOR || value == CELL_TYPE_CVAT_INFO || value == CELL_TYPE_CVAT_PREPROCESSOR || value == CELL_TYPE_CVAT_POSTPROCESSOR) {
+        if(value == CELL_TYPE_PREDICT || value == CELL_TYPE_CVAT_INFO || value == CELL_TYPE_CVAT_INVOKE) {
             isDuplicated = TagsUtils.isTransformerTagExistedInOtherCells(
                 this.props.notebookPanel,
                 this.props.notebookPanel.content.activeCellIndex,
@@ -487,7 +446,6 @@ export class CellMetadataEditor extends React.Component<IProps, IState> {
     }
 
     render() {
-        const cellTag = RESERVED_CELL_NAMES.includes(this.state.transformerTag)? this.state.transformerTag : 'na';
         const cellColor = 'transparent';
 
         // add class names for styling
@@ -534,7 +492,7 @@ export class CellMetadataEditor extends React.Component<IProps, IState> {
                         <Select
                             updateValue={this.updateCurrentCellTag}
                             values={this.state.typeSelectOptions}
-                            value={cellTag}
+                            value={RESERVED_CELL_NAMES.includes(this.state.transformerTag)? this.state.transformerTag : 'na'}
                             label={'Cell tag'}
                             index={0}
                             variant="outlined"
